module Parse (parseString, parseExpression, parseFile) where

import System.IO
import Control.Monad
import Text.ParserCombinators.Parsec
import Text.ParserCombinators.Parsec.Expr
import Text.ParserCombinators.Parsec.Language
import qualified Text.ParserCombinators.Parsec.Token as Token

import Types

-- You'll find these definitions helpful. Read up on Text.ParserCombinators.Parsec.Token to see what they do
language =
  emptyDef { Token.commentStart    = "/*",
             Token.commentEnd      = "*/",
             Token.commentLine     = "//",
             Token.identStart      = letter,
             Token.identLetter     = alphaNum,
             Token.reservedNames   = [ "if", "then", "else", "while", "do",
                                       "and", "or", "not", "true", "false", "print"],
             Token.reservedOpNames = ["+", "-", "*", "/", ":=", "==", "<", ">", "and", "or", "not"]
           }

identifier = Token.identifier lexer
reserved   = Token.reserved   lexer
reservedOp = Token.reservedOp lexer
parens     = Token.parens     lexer
braces     = Token.braces     lexer
integer    = Token.integer    lexer
semi       = Token.semi       lexer
whiteSpace = Token.whiteSpace lexer
commaSep   = Token.commaSep   lexer

lexer = Token.makeTokenParser language

-- This is the parser that will be used in the final parseString function
parser :: Parser Statement
-- You write this one.

-- Parses an expression
expression :: Parser Expression
-- You write this one.

parseString :: String -> Statement
parseString str =
  case parse parser "" str of
      Left e  -> error $ show e
      Right r -> r

parseExpression :: String -> Expression
parseExpression str =
  case parse expression "" str of
      Left e  -> error $ show e
      Right r -> r

parseFile :: String -> IO Statement
parseFile file =
    do result <- parseFromFile parser file
       case result of
           Left e  -> error $ show e
           Right r -> return r
